from numpy import pi
from qosm_core import Vec3

from qosm.utils.Pose import Vector, Frame
from qosm.utils.Field import Field


class Source:
    """Source

    Attributes
    ----------
    TYPE: str
        Typename of the source
    frame: Frame
        The source frame w.r.t. the reference frame (usually world)
    freq: float
        [Hz] Source's frequency
    k0: float
        [rad/meters] Free-space wave number
    lambda0: float
        [meter] Free-space wavelength
    """

    TYPE = '_NONE_'
    DISABLE_MT = False

    def __init__(self, frequency_GHz: float):
        """
        Parameters
        ----------
        frequency_GHz: float
            [GHz] Frequency
        """
        self.frame = Frame()

        # medium and wave parameters
        self.freq = frequency_GHz*1e9
        self.k0 = 2*pi * self.freq / 299792458

        # free-space wavelength
        self.lambda0 = 299792458 / self.freq

    def init(self, **kwargs):
        pass

    def e_field(self, r_pts_src: Vector, pbar=None):
        return Field()

    def h_field(self, r_pts_src: Vector, pbar=None):
        return Field()

    def fields(self, r_pts_src: Vector, pbar=None):
        return Field(), Field(r_pts_src.shape[0]), Vector()

    def oriented_fields(self, r_pts_src: Vector, z_probe_src: Vec3, pbar=None):
        return Field(), Field(), Vector()
